/*
 * TextSuite Java Wrapper by Markus Dllinger
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * 
 * TextSuite (C) Steffen Xonna (aka Lossy eX)
 * http://www.opengl24.de/
 * 
 * -----------------------------------------------------
 * For copyright information, see the file copyright.txt
 * 
 */
package textsuite;

import java.nio.ByteBuffer;
import java.nio.IntBuffer;


/**
 * @author Markus
 * @version 0.8.1
 */
public abstract class TextSuite {


	// Booleans
	public static final int TS_TRUE = 1;
	public static final int TS_FALSE = 0;

	// Inits
	public static final int TS_INIT_TEXTSUITE = 0x01;
	public static final int TS_INIT_SDL_TTF = 0x02;
	public static final int TS_INIT_SDL_IMAGE = 0x04;
	public static final int TS_INIT_GDI = 0x08;
	public static final int TS_INIT_OPENGL = 0x10;
	//public static final int TS_INIT_FREETYPE2 = 0x20

	// library infos
	public static final int TS_INFO_MAYOR_VERSION = 0x01;
	public static final int TS_INFO_MINOR_VERSION = 0x02;
	public static final int TS_INFO_BUILD_NUMBER = 0x03;
	public static final int TS_INFO_VERSION = 0x04;
	public static final int TS_INFO_COPYRIGHT = 0x05;

	public static final int TS_DEFAULT = 0;

	public static final int TS_GLOBAL_ANTIALIASING = 0x11;
	public static final int TS_GLOBAL_FORMAT = 0x12;

	public static final int TS_CONTEXT_BINDING = 0x40;

	// Renderer
	public static final int TS_RENDERER = 0x50;
	public static final int TS_RENDERER_NULL = 0x51;
	public static final int TS_RENDERER_OPENGL = 0x52;

	public static final int TS_RENDERER_NULL_SAVE_IMAGES = 0x60;

	public static final int TS_RENDERER_OPENGL_TEXTURE_SIZE = 0x70;
	public static final int TS_RENDERER_OPENGL_TEXTURE_SIZE_128 = 0x71;
	public static final int TS_RENDERER_OPENGL_TEXTURE_SIZE_256 = 0x72;
	public static final int TS_RENDERER_OPENGL_TEXTURE_SIZE_512 = 0x73;
	public static final int TS_RENDERER_OPENGL_TEXTURE_SIZE_1024 = 0x74;

	// Font
	public static final int TS_FONT_BINDING = 0x100;
	public static final int TS_FONT_COPYRIGHT = 0x101;
	public static final int TS_FONT_FACE_NAME = 0x102;
	public static final int TS_FONT_STYLE_NAME = 0x103;
	public static final int TS_FONT_FULL_NAME = 0x104;
	public static final int TS_FONT_SIZE = 0x105;
	public static final int TS_FONT_STYLE = 0x106;
	public static final int TS_FONT_FILE_STYLE = 0x107;
	public static final int TS_FONT_FIXED_WIDTH = 0x108;
	public static final int TS_FONT_ANTIALIASING = 0x109;
	public static final int TS_FONT_FORMAT = 0x10A;
	public static final int TS_FONT_ASCENT = 0x10B;
	public static final int TS_FONT_DESCENT = 0x113;
	public static final int TS_FONT_EXTERNAL_LEADING = 0x115;
	public static final int TS_FONT_LINESKIP = 0x10C;
	public static final int TS_FONT_CHAR_SPACING = 0x10D;
	public static final int TS_FONT_LINE_SPACING = 0x10E;
	public static final int TS_FONT_UNDERLINE_POSITION = 0x10F;
	public static final int TS_FONT_UNDERLINE_SIZE = 0x110;
	public static final int TS_FONT_STRIKEOUT_POSITION = 0x111;
	public static final int TS_FONT_STRIKEOUT_SIZE = 0x112;
	public static final int TS_FONT_BASELINE_OFFSET = 0x115;

	// Char parameters
	public static final int TS_CHAR_ADVANCE = 0x131;
	public static final int TS_CHAR_GLYPHORIGIN = 0x132;
	public static final int TS_CHAR_GLYPHORIGIN_X = 0x133;
	public static final int TS_CHAR_GLYPHORIGIN_Y = 0x134;
	public static final int TS_CHAR_GLYPHRECT = 0x135;
	public static final int TS_CHAR_GLYPHRECT_TOP = 0x136;
	public static final int TS_CHAR_GLYPHRECT_LEFT = 0x137;
	public static final int TS_CHAR_GLYPHRECT_RIGHT = 0x138;
	public static final int TS_CHAR_GLYPHRECT_BOTTOM = 0x139;

	// Creator
	public static final int TS_CREATOR = 0x200;
	public static final int TS_CREATOR_SDL = 0x201;
	public static final int TS_CREATOR_GDI = 0x202;
	public static final int TS_CREATOR_GDI_FACENAME = 0x203;
	//public static final int TS_CREATOR_FREETYPE2 = 0x204;

	public static final int TS_CREATOR_CREATE_CHARS = 0x210;
	public static final int TS_CREATOR_ADD_RESIZING_BORDER = 0x211;

	// Font Style
	public static final int TS_STYLE_NORMAL = 0x00;
	public static final int TS_STYLE_BOLD = 0x01;
	public static final int TS_STYLE_ITALIC = 0x02;
	public static final int TS_STYLE_UNDERLINE = 0x04;

	// AntiAliasing
	public static final int TS_ANTIALIASING_NONE = 0x221;
	public static final int TS_ANTIALIASING_NORMAL = 0x222;

	// Formats
	public static final int TS_FORMAT_EMPTY = 0x231;
	public static final int TS_FORMAT_RGBA8 = 0x232;

	// PostProcessing usage type
	public static final int TS_POST_INDEX_ALL = -2;
	public static final int TS_POST_INDEX_LAST = -1;
	public static final int TS_POST_USAGE_INCLUDE = 0x291;
	public static final int TS_POST_USAGE_EXCLUDE = 0x292;

	// output consts
	// Single line
	public static final int TS_SINGLE_LINE = 0x300;
	public static final int TS_SINGLE_LINE_TOP = 0x301;
	public static final int TS_SINGLE_LINE_BASELINE = 0x302;

	// Textalign
	public static final int TS_ALIGN = 0x310;
	public static final int TS_ALIGN_LEFT = 0x311;
	public static final int TS_ALIGN_CENTER = 0x312;
	public static final int TS_ALIGN_RIGHT = 0x313;
	public static final int TS_ALIGN_BLOCK = 0x314;

	public static final int TS_VALIGN = 0x320;
	public static final int TS_VALIGN_TOP = 0x321;
	public static final int TS_VALIGN_CENTER = 0x322;
	public static final int TS_VALIGN_BOTTOM = 0x323;

	// Textblockflags
	public static final int TS_BLOCK_OFFSET_X = 0x331;
	public static final int TS_BLOCK_OFFSET_Y = 0x332;

	public static final int TS_BLOCKFLAG_NONE = 0x00;
	public static final int TS_BLOCKFLAG_NO_CLIP = 0x01;
	public static final int TS_BLOCKFLAG_CALC_SIZE = 0x02;
	public static final int TS_BLOCKFLAG_WORD_WRAP = 0x04;
	//public static final int TS_BF_SINGLE_LINE = 0x08;
	//public static final int TS_BF_END_ELLIPSIS = 0x10;

	public static final int TS_CLIP = 0x340;
	public static final int TS_CLIP_COMPLETE = 0x341;
	public static final int TS_CLIP_BORDER = 0x342;

	/*
	public static final int TS_TAB = 0x471;
	public static final int TS_TAB_FIXED = 0x472;
	public static final int TS_TAB_ABSOLUTE = 0x473;
	public static final int TS_TAB_FIXED_WIDTH = 0x474;
	public static final int TS_TAB_ABSOLUTE_POSITIONS = 0x475;
	 */

	// Code pages
	public static final int TS_EMPTY_CP_ENTRY = 0x1000;
	public static final int TS_EMPTY_CP_ENTRY_IGNORE = 0x1001;
	public static final int TS_EMPTY_CP_ENTRY_USE_DEFAULT = 0x1002;

	public static final int TS_CODEPAGE = 0x1100;
	public static final int TS_CODEPAGE_UTF8 = TS_CODEPAGE + 1;
	public static final int TS_CODEPAGE_8859_1 = TS_CODEPAGE + 11;
	public static final int TS_CODEPAGE_8859_2 = TS_CODEPAGE + 12;
	public static final int TS_CODEPAGE_8859_3 = TS_CODEPAGE + 13;
	public static final int TS_CODEPAGE_8859_4 = TS_CODEPAGE + 14;
	public static final int TS_CODEPAGE_8859_5 = TS_CODEPAGE + 15;
	public static final int TS_CODEPAGE_8859_6 = TS_CODEPAGE + 16;
	public static final int TS_CODEPAGE_8859_7 = TS_CODEPAGE + 17;
	public static final int TS_CODEPAGE_8859_8 = TS_CODEPAGE + 18;
	public static final int TS_CODEPAGE_8859_9 = TS_CODEPAGE + 19;
	public static final int TS_CODEPAGE_8859_10 = TS_CODEPAGE + 20;
	public static final int TS_CODEPAGE_8859_11 = TS_CODEPAGE + 21;
	public static final int TS_CODEPAGE_8859_13 = TS_CODEPAGE + 22;
	public static final int TS_CODEPAGE_8859_14 = TS_CODEPAGE + 23;
	public static final int TS_CODEPAGE_8859_15 = TS_CODEPAGE + 24;
	public static final int TS_CODEPAGE_8859_16 = TS_CODEPAGE + 25;
	public static final int TS_CODEPAGE_037 = TS_CODEPAGE + 31;
	public static final int TS_CODEPAGE_437 = TS_CODEPAGE + 32;
	public static final int TS_CODEPAGE_500 = TS_CODEPAGE + 33;
	public static final int TS_CODEPAGE_737 = TS_CODEPAGE + 34;
	public static final int TS_CODEPAGE_775 = TS_CODEPAGE + 35;
	public static final int TS_CODEPAGE_850 = TS_CODEPAGE + 36;
	public static final int TS_CODEPAGE_852 = TS_CODEPAGE + 37;
	public static final int TS_CODEPAGE_855 = TS_CODEPAGE + 38;
	public static final int TS_CODEPAGE_857 = TS_CODEPAGE + 39;
	public static final int TS_CODEPAGE_860 = TS_CODEPAGE + 40;
	public static final int TS_CODEPAGE_861 = TS_CODEPAGE + 41;
	public static final int TS_CODEPAGE_862 = TS_CODEPAGE + 42;
	public static final int TS_CODEPAGE_863 = TS_CODEPAGE + 43;
	public static final int TS_CODEPAGE_864 = TS_CODEPAGE + 44;
	public static final int TS_CODEPAGE_865 = TS_CODEPAGE + 45;
	public static final int TS_CODEPAGE_866 = TS_CODEPAGE + 46;
	public static final int TS_CODEPAGE_869 = TS_CODEPAGE + 47;
	public static final int TS_CODEPAGE_874 = TS_CODEPAGE + 48;
	public static final int TS_CODEPAGE_875 = TS_CODEPAGE + 49;
	public static final int TS_CODEPAGE_1026 = TS_CODEPAGE + 50;
	public static final int TS_CODEPAGE_1250 = TS_CODEPAGE + 51;
	public static final int TS_CODEPAGE_1251 = TS_CODEPAGE + 52;
	public static final int TS_CODEPAGE_1252 = TS_CODEPAGE + 53;
	public static final int TS_CODEPAGE_1253 = TS_CODEPAGE + 54;
	public static final int TS_CODEPAGE_1254 = TS_CODEPAGE + 55;
	public static final int TS_CODEPAGE_1255 = TS_CODEPAGE + 56;
	public static final int TS_CODEPAGE_1256 = TS_CODEPAGE + 57;
	public static final int TS_CODEPAGE_1257 = TS_CODEPAGE + 58;
	public static final int TS_CODEPAGE_1258 = TS_CODEPAGE + 59;

	// bluring masks
	public static final int TS_CHANNEL_RED = 0x01;
	public static final int TS_CHANNEL_GREEN = 0x02;
	public static final int TS_CHANNEL_BLUE = 0x04;
	public static final int TS_CHANNEL_ALPHA = 0x08;
	public static final int TS_CHANNEL_LUMINANCE = 0x10;

	public static final int TS_CHANNELS_RGB = TS_CHANNEL_RED | TS_CHANNEL_GREEN | TS_CHANNEL_BLUE;
	public static final int TS_CHANNELS_RGBA = TS_CHANNELS_RGB | TS_CHANNEL_ALPHA;
	public static final int TS_CHANNELS_LUMINANCE_ALPHA = TS_CHANNEL_LUMINANCE | TS_CHANNEL_ALPHA;

	public static final int TS_IMAGE_RED_MODE = 0x401;
	public static final int TS_IMAGE_GREEN_MODE = 0x402;
	public static final int TS_IMAGE_BLUE_MODE = 0x403;
	public static final int TS_IMAGE_ALPHA_MODE = 0x404;
	public static final int TS_IMAGE_LUMINANCE_MODE = 0x405;

	public static final int TS_MODE_REPLACE = 0x411;
	public static final int TS_MODE_MODULATE = 0x412;

	// Imageloading
	public static final int TS_IMAGE_LOADER = 0x420;
	public static final int TS_IMAGE_LOADER_SDL = 0x421;

	// Errorcode
	public static final int TS_NO_ERROR = 0x0000;
	public static final int TS_ERROR = 0x8000;
	public static final int TS_NO_ACTIVE_CONTEXT = TS_ERROR + 0x1;
	public static final int TS_NO_ACTIVE_RENDERER = TS_ERROR + 0x2;
	public static final int TS_NO_ACTIVE_FONT = TS_ERROR + 0x3;
	public static final int TS_INVALID_OPERATION = TS_ERROR + 0x4;
	public static final int TS_INVALID_ENUM = TS_ERROR + 0x5;
	public static final int TS_INVALID_VALUE = TS_ERROR + 0x6;
	public static final int TS_OUT_OF_MEMORY = TS_ERROR + 0x7;
	public static final int TS_NOT_INITIALIZED = TS_ERROR + 0x8;

	public static final int TS_NO_FUNC = 0x0000;
	public static final int TS_FUNC = 0x4000;
	public static final int TS_FUNC_INIT = TS_FUNC + 0x01;
	public static final int TS_FUNC_QUIT = TS_FUNC + 0x02;
	public static final int TS_FUNC_SET_PARAMETER = TS_FUNC + 0x03;
	public static final int TS_FUNC_GET_PARAMETER = TS_FUNC + 0x04;
	public static final int TS_FUNC_GET_STRING = TS_FUNC + 0x05;
	public static final int TS_FUNC_CONTEXT_CREATE = TS_FUNC + 0x06;
	public static final int TS_FUNC_CONTEXT_DESTROY = TS_FUNC + 0x07;
	public static final int TS_FUNC_CONTEXT_BIND = TS_FUNC + 0x08;
	public static final int TS_FUNC_FONT_CREATE_CREATOR = TS_FUNC + 0x09;
	public static final int TS_FUNC_FONT_DESTROY = TS_FUNC + 0x0A;
	public static final int TS_FUNC_FONT_BIND = TS_FUNC + 0x0B;
	public static final int TS_FUNC_FONT_ADD_CHAR = TS_FUNC + 0x0C;
	public static final int TS_FUNC_FONT_DELETE_CHAR = TS_FUNC + 0x0D;
	public static final int TS_FUNC_FONT_SET_CHAR_PARAMETER = TS_FUNC + 0x0E;
	public static final int TS_FUNC_FONT_GET_CHAR_PARAMETER = TS_FUNC + 0x0F;
	public static final int TS_FUNC_POST_ADD_FILL_COLOR = TS_FUNC + 0x10;
	public static final int TS_FUNC_POST_ADD_FILL_PATTERN = TS_FUNC + 0x11;
	public static final int TS_FUNC_POST_ADD_BORDER = TS_FUNC + 0x12;
	public static final int TS_FUNC_POST_ADD_SHADOW = TS_FUNC + 0x13;
	public static final int TS_FUNC_POST_ADD_CUSTOM = TS_FUNC + 0x14;
	public static final int TS_FUNC_POST_DELETE = TS_FUNC + 0x15;
	public static final int TS_FUNC_POST_ADD_USAGE = TS_FUNC + 0x16;
	public static final int TS_FUNC_POST_CLEAR_USAGE = TS_FUNC + 0x17;
	public static final int TS_FUNC_STRING_ANSI_TO_WIDE = TS_FUNC + 0x18;
	public static final int TS_FUNC_STRING_ALLOC = TS_FUNC + 0x19;
	public static final int TS_FUNC_STRING_DISPOSE = TS_FUNC + 0x1A;
	public static final int TS_FUNC_TEXT_BEGIN_BLOCK = TS_FUNC + 0x1B;
	public static final int TS_FUNC_TEXT_END_BLOCK = TS_FUNC + 0x1C;
	public static final int TS_FUNC_TEXT_COLOR = TS_FUNC + 0x1D;
	public static final int TS_FUNC_TEXT_OUT = TS_FUNC + 0x1E;
	public static final int TS_FUNC_TEXT_GET_WIDTH = TS_FUNC + 0x1F;
	public static final int TS_FUNC_TEXT_GET_HEIGHT = TS_FUNC + 0x20;
	public static final int TS_FUNC_IMAGE_CREATE = TS_FUNC + 0x21;
	public static final int TS_FUNC_IMAGE_DESTROY = TS_FUNC + 0x22;
	public static final int TS_FUNC_IMAGE_LOAD = TS_FUNC + 0x23;
	public static final int TS_FUNC_IMAGE_ASSIGN_FROM = TS_FUNC + 0x24;
	public static final int TS_FUNC_IMAGE_NEW = TS_FUNC + 0x25;
	public static final int TS_FUNC_IMAGE_GET_INFO = TS_FUNC + 0x26;
	public static final int TS_FUNC_IMAGE_SCANLINE = TS_FUNC + 0x27;
	public static final int TS_FUNC_IMAGE_RESIZE = TS_FUNC + 0x28;
	public static final int TS_FUNC_IMAGE_BLEND = TS_FUNC + 0x29;
	public static final int TS_FUNC_IMAGE_BLUR = TS_FUNC + 0x2A;
	public static final int TS_FUNC_IMAGE_FILL_COLOR = TS_FUNC + 0x2B;
	public static final int TS_FUNC_IMAGE_FILL_PATTERN = TS_FUNC + 0x2C;



	// global functions
	private native static int ntsInit(int Names);
	private native static int ntsWasInit(int Names);
	private native static void ntsQuit();

	private native static void ntsSetParameteri(int ParamName, int Param);
	private native static int ntsGetParameteri(int ParamName);
	private native static String ntsGetString(int ParamName);

	private native static int ntsGetError();
	private native static int ntsGetErrorFunction();
	private native static String ntsGetErrorString(int ErrorCode);

	// context functions
	private native static int ntsContextCreate();
	private native static void ntsContextDestroy(int ContextID);
	private native static void ntsContextBind(int ContextID);

	// font functions
	private native static int ntsFontCreateCreatorA(String Name, int Size, int Style, int AntiAliasing, int Format);
	private native static void ntsFontDestroy(int FontID);
	private native static void ntsFontBind(int FontID);

	private native static void ntsFontAddCharRange(char CharStart, char CharEnd);
	private native static void ntsFontAddChars(char[] Chars);
	private native static void ntsFontAddChar(char Char);

	private native static void ntsFontDeleteCharRange(char CharStart, char CharEnd);
	private native static void ntsFontDeleteChars(char[] Chars);
	private native static void ntsFontDeleteChar(char Char);

	private native static void ntsFontSetCharParameteri(char Char, int ParamName, int Param);
	private native static void ntsFontSetCharParameteriv(char Char, int ParamName, IntBuffer buffer, int buffer_position);
	private native static int ntsFontGetCharParameteri(char Char, int ParamName);
	private native static void ntsFontGetCharParameteriv(char Char, int ParamName, IntBuffer buffer, int buffer_position);

	private native static void ntsPostAddFillColor3f(float Red, float Green, float Blue, int ChannelMask);
	private native static void ntsPostAddFillColor4f(float Red, float Green, float Blue, float Alpha, int ChannelMask);

	private native static void ntsPostAddFillPattern(int PatternImageID, int X, int Y, int ChannelMask);

	private native static void ntsPostAddBorder3f(float Width, float Strength, float Red, float Green, float Blue);
	private native static void ntsPostAddBorder4f(float Width, float Strength, float Red, float Green, float Blue, float Alpha);

	private native static void ntsPostAddShadow3f(float Radius, int X, int Y, float Red, float Green, float Blue);
	private native static void ntsPostAddShadow4f(float Radius, int X, int Y, float Red, float Green, float Blue, float Alpha);

	//private native static void ntsPostAddKerning ();
	private native static void ntsPostAddCustom(IPostProcessProc post, Object Data);

	private native static void ntsPostDelete(int PostIndex);

	private native static void ntsPostAddUsageRange(int PostIndex, int UsageType, char CharStart, char CharEnd);
	private native static void ntsPostAddUsageChars(int PostIndex, int UsageType, char[] Chars);

	private native static void ntsPostClearUsage (int PostIndex);

	// drawing functions
	private native static void ntsTextBeginBlock(int Left, int Top, int Width, int Height, int flags);
	private native static void ntsTextEndBlock();

	private native static void ntsTextColor3f(float Red, float Green, float Blue);
	private native static void ntsTextColor4f(float Red, float Green, float Blue, float Alpha);

	private native static void ntsTextOut(String Text);
	private native static int ntsTextGetWidth(String Text);
	private native static int ntsTextGetHeight(String Text);

	private native static void ntsCharOut(String CharCode);


	// Image functions
	private native static int ntsImageCreate();
	private native static void ntsImageDestroy(int ImageID);
	private native static void ntsImageLoad(int ImageID, String Filename);
	private native static void ntsImageAssignFrom(int ImageID, int FromImageID);
	private native static void ntsImageNew(int ImageID, int Width, int Height, int Format);

	//private native static void ntsImageGetInfo(int ImageID, IntBuffer buffer, int buffer_position, ByteBuffer data, int data_position);
	private native static int ntsImageGetIsEmpty(int ImageID);
	private native static int ntsImageGetWidth(int ImageID);
	private native static int ntsImageGetHeight(int ImageID);
	private native static int ntsImageGetFormat(int ImageID);
	private native static ByteBuffer ntsImageGetData(int ImageID);
	private native static ByteBuffer ntsImageScanline(int ImageID, int Scanline);

	private native static void ntsImageResize(int ImageID, int Width, int Height, int X, int Y);
	private native static void ntsImageBlend(int ImageID, int OverImageID, int X, int Y, int AutoExpand);
	private native static void ntsImageBlur(int ImageID, float X, float Y, int ChannelMask, int AutoExpand, IntBuffer buffer, int buffer_position);

	private native static void ntsImageFillColor3f(int ImageID, float Red, float Green, float Blue, int ChannelMask);
	private native static void ntsImageFillColor4f(int ImageID, float Red, float Green, float Blue, float Alpha, int ChannelMask);

	private native static void ntsImageFillPattern(int ImageID, int PatternImageID, int X, int Y, int ChannelMask); 


	/*
	 * Load the required libraries. Note that both have to be in the Java
	 * library path.
	 */
	static {
		System.loadLibrary("libTextSuite");
		System.loadLibrary("TextSuite");
	}


	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
	 * 										global functions
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

	/**
	 * <p> Initializes the TextSuite modules specified in the Names argument.
	 * Names is a bitmask of one or more of the following constants, conjuncted
	 * with the bitwise or operator.</p>
	 * <ul>
	 * 	<li>TS_INIT_TEXTSUITE</li>
	 * 	<li>TS_INIT_SDL_TTF</li>
	 * 	<li>TS_INIT_SDL_IMAGE</li>
	 * 	<li>TS_INIT_GDI</li>
	 * 	<li>TS_INIT_OPENGL</li>
	 * 	<li>TS_INIT_FREETYPE2</li>
	 * </ul>
	 * <p>To query which modules were already initialized, call {@link #tsWasInit()}</p>
	 * 
	 * @param Names		The module bitmask.
	 * @return			<code>true</code> if the initialization was
	 * 					successful
	 */
	public static boolean tsInit(int Names) {
		return ntsInit(Names) == TS_TRUE;
	}

	/**
	 * <p>Queries which modules were initialized. For a list of modules, see
	 * {@link #tsInit(int)}</p>
	 * 
	 * @param Names		The module bitmask.
	 * @return			<code>true</code> if all modules in the bitmask were initialized.
	 */
	public static int tsWasInit(int Names) {
		return ntsWasInit(Names);
	}

	public static void tsQuit() {
		ntsQuit();
	}

	public static int tsGetError() {
		return ntsGetError();
	}

	public static int tsGetErrorFunction()
	{	return ntsGetErrorFunction();
	}

	public static String tsGetErrorString(int ErrorCode) {
		return ntsGetErrorString(ErrorCode);
	}



	public static void tsSetParameteri(int ParamName, int Param) {
		ntsSetParameteri(ParamName, Param);
	}


	public static int tsGetParameteri(int ParamName) {
		return ntsGetParameteri(ParamName);
	}


	public static String tsGetString(int ParamName) {
		return ntsGetString(ParamName);
	}


	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
	 * 										context functions
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

	public static int tsContextCreate() {
		return ntsContextCreate();
	}

	public static void tsContextDestroy(int ContextID) {
		ntsContextDestroy(ContextID);
	}

	public static void tsContextBind(int ContextID) {
		ntsContextBind(ContextID);
	}


	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
	 * 										font functions
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

	public static int tsFontCreateCreatorA(String Name, int Size, int Style, int AntiAliasing, int Format) {
		return ntsFontCreateCreatorA(Name, Size, Style, AntiAliasing, Format);
	}

	public static void tsFontDestroy(int FontID) {
		ntsFontDestroy(FontID);
	}

	public static void tsFontBind(int FontID){
		ntsFontBind(FontID);
	}

	public static void tsFontAddCharRange(char CharStart, char CharEnd) {
		ntsFontAddCharRange(CharStart, CharEnd);
	}

	public static void tsFontAddChars(char[] Chars) {
		if (Chars[Chars.length - 1] != 0x0) {
			char[] _Chars = new char[Chars.length + 1];
			System.arraycopy(Chars, 0, _Chars, 0, Chars.length);
			_Chars[_Chars.length - 1] = 0x0;
			ntsFontAddChars(_Chars);
		} else {
			ntsFontAddChars(Chars);
		}
	}

	public static void tsFontAddChar(char Char) {
		ntsFontAddChar(Char);
	}


	public static void tsFontDeleteCharRange(char CharStart, char CharEnd) {
		ntsFontDeleteCharRange(CharStart, CharEnd);
	}

	public static void tsFontDeleteChars(char[] Chars) {
		if (Chars[Chars.length - 1] != 0x0) {
			char[] _Chars = new char[Chars.length + 1];
			System.arraycopy(Chars, 0, _Chars, 0, Chars.length);
			_Chars[_Chars.length - 1] = 0x0;
			ntsFontDeleteChars(_Chars);
		} else {
			ntsFontDeleteChars(Chars);
		}
	}

	public static void tsFontDeleteChar(char Char) {
		ntsFontDeleteChar(Char);
	}



	public static void tsFontSetCharParameteriv(char Char, int ParamName, IntBuffer buffer) {
		ntsFontSetCharParameteriv(Char, ParamName, buffer, buffer.position());
	}

	public static void tsFontSetCharParameteri(char Char, int ParamName, int Param) {
		ntsFontSetCharParameteri(Char, ParamName, Param);
	}

	public static void tsFontGetCharParameteriv(char Char, int ParamName, IntBuffer buffer)
	{
		ntsFontGetCharParameteriv(Char, ParamName, buffer, buffer.position());
	}

	public static int tsFontGetCharParameteri(char Char, int ParamName) {
		return ntsFontGetCharParameteri(Char, ParamName);
	}



	public static void tsPostAddFillColor3f(float Red, float Green, float Blue, int ChannelMask) {
		ntsPostAddFillColor3f(Red, Green, Blue, ChannelMask);
	}

	public static void tsPostAddFillColor4f(float Red, float Green, float Blue, float Alpha, int ChannelMask) {
		ntsPostAddFillColor4f(Red, Green, Blue, Alpha, ChannelMask);
	}

	public static void tsPostAddFillColor3ub(int Red, int Green, int Blue, int ChannelMask) {
		ntsPostAddFillColor3f(Red / 255f, Green / 255f, Blue / 255f, ChannelMask);
	}

	public static void tsPostAddFillColor4ub(int Red, int Green, int Blue, int Alpha, int ChannelMask) {
		ntsPostAddFillColor4f(Red / 255f, Green / 255f, Blue / 255f, Alpha / 255f, ChannelMask);
	}



	public static void tsPostAddFillPattern(int PatternImageID, int X, int Y, int ChannelMask) {
		ntsPostAddFillPattern(PatternImageID, X, Y, ChannelMask);
	}



	public static void tsPostAddBorder3f(float Width, float Strength, float Red, float Green, float Blue) {
		ntsPostAddBorder3f(Width, Strength, Red, Green, Blue);
	}

	public static void tsPostAddBorder4f(float Width, float Strength, float Red, float Green, float Blue, float Alpha) {
		ntsPostAddBorder4f(Width, Strength, Red, Green, Blue, Alpha);
	}

	public static void tsPostAddBorder3ub(float Width, float Strength, int Red, int Green, int Blue) {
		ntsPostAddBorder3f(Width, Strength, Red / 255f, Green / 255f, Blue / 255f);
	}

	public static void tsPostAddBorder4ub(float Width, float Strength, int Red, int Green, int Blue, int Alpha) {
		ntsPostAddBorder4f(Width, Strength, Red / 255f, Green / 255f, Blue / 255f, Alpha / 255f);
	}



	public static void tsPostAddShadow3f(float Radius, int X, int Y, float Red, float Green, float Blue) {
		ntsPostAddShadow3f(Radius, X, Y, Red, Green, Blue);
	}

	public static void tsPostAddShadow4f(float Radius, int X, int Y, float Red, float Green, float Blue, float Alpha) {
		ntsPostAddShadow4f(Radius, X, Y, Red, Green, Blue, Alpha);
	}

	public static void tsPostAddShadow3ub(float Radius, int X, int Y, int Red, int Green, int Blue) {
		ntsPostAddShadow3f(Radius, X, Y, Red / 255f, Green / 255f, Blue / 255f);
	}

	public static void tsPostAddShadow4ub(float Radius, int X, int Y, int Red, int Green, int Blue, int Alpha) {
		ntsPostAddShadow4f(Radius, X, Y, Red / 255f, Green / 255f, Blue / 255f, Alpha / 255f);
	}


	public static void tsPostAddCustom(IPostProcessProc post, Object Data) {
		if (post == null)
			return;
		ntsPostAddCustom(post, Data);
	}


	public static void tsPostDelete(int PostIndex) {
		ntsPostDelete(PostIndex);
	}

	public static void tsPostAddUsageRange(int PostIndex, int UsageType, char CharStart, char CharEnd) {
		ntsPostAddUsageRange(PostIndex, UsageType, CharStart, CharEnd);
	}

	public static void tsPostAddUsageChars(int PostIndex, int UsageType, char[] Chars) {
		if (Chars[Chars.length - 1] != 0x0) {
			char[] _Chars = new char[Chars.length + 1];
			System.arraycopy(Chars, 0, _Chars, 0, Chars.length);
			_Chars[_Chars.length - 1] = 0x0;
			ntsPostAddUsageChars(PostIndex, UsageType, _Chars);
		} else {
			ntsPostAddUsageChars(PostIndex, UsageType, Chars);
		}
	}


	public static void tsPostClearUsage (int PostIndex) {
		ntsPostClearUsage(PostIndex);
	}



	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
	 * 										drawing functions
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

	public static void tsTextBeginBlock(int Left, int Top, int Width, int Height, int flags) {
		ntsTextBeginBlock(Left, Top, Width, Height, flags);
	}

	public static void tsTextEndBlock() {
		ntsTextEndBlock();	
	}

	public static void tsTextColor3f(float Red, float Green, float Blue) {
		ntsTextColor3f(Red, Green, Blue);
	}

	public static void tsTextColor4f(float Red, float Green, float Blue, float Alpha) {
		ntsTextColor4f(Red, Green, Blue, Alpha);
	}

	public static void tsTextColor3ub(int Red, int Green, int Blue) {
		ntsTextColor3f(Red / 255f, Green / 255f, Blue / 255f);
	}

	public static void tsTextColor4ub(int Red, int Green, int Blue, int Alpha) {
		ntsTextColor4f(Red / 255f, Green / 255f, Blue / 255f, Alpha / 255f);
	}

	public static void tsTextOut(String Text) {
		if (Text != null)
			ntsTextOut(Text);
	}

	public static int tsTextGetWidth(String Text) {
		return ntsTextGetWidth(Text);
	}

	public static int tsTextGetHeight(String Text) {
		return ntsTextGetHeight(Text);
	}

	public static void tsCharOut(char CharCode) {
		ntsCharOut(String.valueOf(CharCode));
	}


	// Image functions
	public static int tsImageCreate() {
		return ntsImageCreate();
	}

	public static void tsImageDestroy(int ImageID) {
		ntsImageDestroy(ImageID);
	}

	public static void tsImageLoad(int ImageID, String Filename) {
		ntsImageLoad(ImageID, Filename);
	}

	public static void tsImageAssignFrom(int ImageID, int FromImageID) {
		ntsImageAssignFrom(ImageID, FromImageID);
	}

	public static void tsImageNew(int ImageID, int Width, int Height, int Format) {
		ntsImageNew(ImageID, Width, Height, Format);
	}

	public static int tsImageGetSize(int ImageID) {
		int width = tsImageGetWidth(ImageID);
		int height = tsImageGetHeight(ImageID);
		int format = tsImageGetFormat(ImageID);

		int size = width * height;
		switch (format) {
		case TS_FORMAT_RGBA8:
			return size * 4;
		case TS_FORMAT_EMPTY:
			return 0;
		}	
		return 0;
	}

	/**
	 * Retrieves image information. The values are being stored
	 * in a new {@link ImageGetInfoArguments} object. For further
	 * information regarding the image data, see {@link #tsImageGetData(int))}.
	 * 
	 * @param ImageID		The id of the image.
	 * @return				The image information.
	 * 
	 * @see ImageGetInfoArguments
	 */
	public static ImageGetInfoArguments tsImageGetInfo(int ImageID) {

		//TODO: Modify the native function so that the image buffer
		//		is returned and the pointer arguments are stored in
		//		a byte buffer. Afterwards move the data into the
		//		arguments object.

		ImageGetInfoArguments arguments = new ImageGetInfoArguments();

		int width = tsImageGetWidth(ImageID);
		int height = tsImageGetHeight(ImageID);
		int format = tsImageGetFormat(ImageID);

		arguments.Width = width;
		arguments.Height = height;
		arguments.Format = format;
		arguments.data = null;

		int size = width * height;
		switch (format) {
		case TS_FORMAT_RGBA8:
			size *= 4;
			break;
		case TS_FORMAT_EMPTY:
			return arguments;
		}

		arguments.data = ntsImageGetData(ImageID);

		return arguments;
	}

	public static int tsImageGetIsEmpty(int ImageID) {
		return ntsImageGetIsEmpty(ImageID);
	}

	public static int tsImageGetWidth(int ImageID) {
		return ntsImageGetWidth(ImageID);
	}

	public static int tsImageGetHeight(int ImageID) {
		return ntsImageGetHeight(ImageID);
	}

	public static int tsImageGetFormat(int ImageID) {
		return ntsImageGetFormat(ImageID);
	}

	/**
	 * <p>Returns a ByteBuffer containing the data of the specified
	 * image, or <code>null</code>, if the id is invalid.<p>
	 * 
	 * <p>Operations on the buffer will directly affect the image data
	 * in the TextSuite library.</p>
	 * 
	 * @param ImageID		The id of the image.
	 * @return				A ByteBuffer containing the
	 * 						image data.
	 */
	public static ByteBuffer tsImageGetData(int ImageID) {
		return ntsImageGetData(ImageID);
	}

	/**
	 * Returns a ByteBuffer containing the entire image data.
	 * The position is set to the beginning of the line specified
	 * by Scanline. For further information regarding the image
	 * data, see {@link #tsImageGetData(int)}.
	 * 
	 * @param ImageID		The id of the image.
	 * @param Scanline		The line index.
	 * @return				A ByteBuffer containing the entire
	 * 						image data. The position is set to
	 * 						the specified line.
	 * 
	 * @see ImageGetInfoArguments
	 */
	public static ByteBuffer tsImageScanline(int ImageID, int Scanline) {
		//TODO: The native tsImageScanline function is the same as 
		//		tsImageGetData. That is because we want the buffer
		//      to contain the entire data of the image. Otherwise
		//		leaving the bounds of the line specified by Scanline
		//		would case an error.
		ByteBuffer buffer = ntsImageScanline(ImageID, Scanline);
		int lineSize = tsImageGetWidth(ImageID) * ((tsImageGetFormat(ImageID) == TS_FORMAT_EMPTY) ? 0 : 4);
		buffer.position(lineSize * Scanline);
		return buffer;
	}


	public static void tsImageResize(int ImageID, int Width, int Height, int X, int Y) {
		ntsImageResize(ImageID, Width, Height, X, Y);
	}

	public static void tsImageBlend(int ImageID, int OverImageID, int X, int Y, boolean AutoExpand) {
		ntsImageBlend(ImageID, OverImageID, X, Y, AutoExpand ? TS_TRUE : TS_FALSE);
	}

	public static ImageBlurArguments tsImageBlur(int ImageID, float X, float Y, int ChannelMask, boolean AutoExpand) {
		ImageBlurArguments arguments = new ImageBlurArguments();
		IntBuffer buf = BufferUtils.createIntBuffer(2);
		ntsImageBlur(ImageID, X, Y, ChannelMask, AutoExpand ? TS_TRUE : TS_FALSE, buf, buf.position());
		arguments.expandSizeX = buf.get();
		arguments.expandSizeY = buf.get();
		return arguments;
	}


	public static void tsImageFillColor3f(int ImageID, float Red, float Green, float Blue, int ChannelMask) {
		ntsImageFillColor3f(ImageID, Red, Green, Blue, ChannelMask);
	}

	public static void tsImageFillColor3ub(int ImageID, int Red, int Green, int Blue, int ChannelMask) {
		ntsImageFillColor3f(ImageID, Red / 255f, Green / 255f, Blue / 255f, ChannelMask);
	}

	public static void tsImageFillColor4f(int ImageID, float Red, float Green, float Blue, float Alpha, int ChannelMask) {
		ntsImageFillColor4f(ImageID, Red, Green, Blue, Alpha, ChannelMask);
	}

	public static void tsImageFillColor4ub(int ImageID, int Red, int Green, int Blue, int Alpha, int ChannelMask) {
		ntsImageFillColor4f(ImageID, Red / 255f, Green / 255f, Blue / 255f, Alpha / 255f, ChannelMask);
	}


	public static void tsImageFillPattern(int ImageID, int PatternImageID, int X, int Y, int ChannelMask) {
		ntsImageFillPattern(ImageID, PatternImageID, X, Y, ChannelMask);
	}

}
